package com.mt.wms.empty.controller;

import com.mt.wms.core.base.BaseController;
import com.mt.wms.core.constants.CommonConstant;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.params.CurrTaskQueryParam;
import com.mt.wms.empty.params.TaskCreateParam;
import com.mt.wms.empty.service.CurrTaskDetService;
import com.mt.wms.empty.service.CurrTaskService;
import com.mt.wms.empty.service.OrderInfoService;
import com.mt.wms.empty.vo.CurrTaskMainQueryVo;
import com.mt.wms.empty.vo.CurrTaskQueryVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@RestController
@RequestMapping(CommonConstant.API_MODULE_BASE + "currTask")
@Slf4j
@Api(value = "任务管理", tags = "当前任务管理", hidden = false)
public class CurrTaskController extends BaseController {
    @Autowired
    private CurrTaskService currTaskService;
    @Autowired
    private CurrTaskDetService currTaskDetService;
    @Autowired
    private OrderInfoService orderInfoService;

    @PostMapping(value = "createProcessTask")
    @ApiOperation(value = "订单加工-创建一个加工任务至任务队列")
    public R<String> createProcessTask(@Validated @RequestBody TaskCreateParam param) {
        //验证标识卡号正确无误。
        orderInfoService.verifyTaskInfoByIdenCardNum(param.getDetParams());
        R<IdVo> mainTask = currTaskService.createProcessTask(param);
        currTaskDetService.createProcessTaskDet(param.getDetParams(), mainTask.getData().getId());
        return successful("创建成功，任务添加至等待执行队列。");
    }


    @PostMapping(value = "mainCurrentTaskNow")
    @ApiOperation(value = "首页-获取当前执行中的任务")
    public R<PageVo<CurrTaskMainQueryVo>> mainCurrentTaskNow(BasePageParam param) {
        return currTaskService.currentTaskMainPage(param);
    }

    @PostMapping(value = "currentTaskNow")
    @ApiOperation(value = "获取当前执行的任务 (车辆名称传id)")
    public R<PageVo<CurrTaskQueryVo>> currentTask(@Validated @RequestBody CurrTaskQueryParam param) {
        return currTaskService.currentTaskPage(param);
    }

    @PostMapping(value = "completeTask")
    @ApiOperation(value = "执行任务完成（仅wms，不通知apms）")
    public R<String> completeTask(@Validated @RequestBody IdParam param) {
        return currTaskService.completeTask(param);
    }

    @PostMapping(value = "deleteBySheetNo")
    @ApiOperation(value = "人工通过生产单编号删除未正式开始的生产单(通知APMS)")
    public R<String> deleteBySheetNo(@Validated @RequestBody String sheetNo) {
        return currTaskService.deleteBySheetNo(sheetNo);
    }
}
