package com.mt.wms.empty.task;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.*;
import com.mt.wms.core.vo.R;
import org.springframework.scheduling.annotation.Async;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;

/**
 * @Author: liguanghao
 * @Date: 2021/11/15 21:55
 * @Version 1.0
 */
@Transactional
public class RunTaskUtils {
    @Resource
    private CurrTaskServiceBiz currTaskServiceBiz;
    @Resource
    private RunTaskServiceBiz runTaskServiceBiz;
    @Resource
    private LocationServiceBiz locationServiceBiz;
    @Resource
    private AsynRunTaskService asynRunTaskService;
    @Resource
    private VehicleInfoServiceBiz vehicleInfoServiceBiz;


    public R runTask(Integer wcsTaskId) throws InterruptedException {
        CurrTask currTask = currTaskServiceBiz.getById(wcsTaskId);
        //传入任务id,起始点，若终点是窑炉，获取窑炉状态
        int kilnStatus = 1;

        //窑炉状态为不可用
        if (kilnStatus == 0) {
            return R.failed("目标窑炉不可用！");
        }
        //窑炉可用未满
        if (kilnStatus==1){
            //更改任务状态为执行中
            currTask.setStatus(1);
            currTask.setUpdateTime(LocalDateTime.now());
            currTaskServiceBiz.updateById(currTask);
            //查询是否有空闲车辆，若有。占用车辆，若无，返回暂无可用车辆
            List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>()
                    .eq(VehicleInfo.STATUS, 0)
                    .eq(VehicleInfo.VALID, 1));
            if (vehicleInfoList.size()>0){
                VehicleInfo vehicleInfo = vehicleInfoList.get(0);
                vehicleInfo.setStatus(1);
                vehicleInfo.setUpdateTime(LocalDateTime.now());
                vehicleInfoServiceBiz.updateById(vehicleInfo);
            }else {
                return R.failed("暂无可用车辆!请稍后重试！");
            }
            //异步调用车辆
            asynRunTaskService.asynRunTask(currTask);
            return R.ok("操作成功");
        }
        //窑炉已满
        if (kilnStatus==2){
            //查询缓存区库位是否已满
            int count = locationServiceBiz.count(new QueryWrapper<Location>()
                    .eq(Location.STATUS, 0)
                    .eq(Location.VALID, 1));
            //缓存区未满
            if (count >0) {
                //更改任务状态为执行中
                currTask.setStatus(1);
                currTask.setUpdateTime(LocalDateTime.now());
                currTaskServiceBiz.updateById(currTask);
                //查询是否有空闲车辆，若有。占用车辆，若无，返回暂无可用车辆
                List<VehicleInfo> vehicleInfoList = vehicleInfoServiceBiz.list(new QueryWrapper<VehicleInfo>()
                        .eq(VehicleInfo.STATUS, 0)
                        .eq(VehicleInfo.VALID, 1));
                if (vehicleInfoList.size()>0){
                    VehicleInfo vehicleInfo = vehicleInfoList.get(0);
                    vehicleInfo.setStatus(1);
                    vehicleInfo.setUpdateTime(LocalDateTime.now());
                    vehicleInfoServiceBiz.updateById(vehicleInfo);
                }else {
                    return R.failed("暂无可用车辆!请稍后重试！");
                }
                //异步调用车辆
                asynRunTaskService.asynRunTaskToWarehouse(currTask);
                return R.ok("操作成功！当前目标窑炉已满，托盘加入缓存区待加工队列。");
            }
            return R.failed("当前目标窑炉已满！缓存区已满！请稍后重试!");
        }
        return null;
    }
}
